#include "vector/vec.h"
#include "math/math_utils.h"
#include <math.h>

typedef struct {
  vec3f_t row0;
  vec3f_t row1;
  vec3f_t row2;
} mat3x3f_t;

vec2i_t vec_add_vec2i_t(vec2i_t v1, vec2i_t v2) {
  return (vec2i_t){.x = v1.x + v2.x, .y = v1.y + v2.y};
}

vec2i_t vec_sub_vec2i_t(vec2i_t v1, vec2i_t v2) {
  return (vec2i_t){.x = v1.x - v2.x, .y = v1.y - v2.y};
}

vec2i_t vec_mul_vec2i_t(vec2i_t v1, vec2i_t v2) {
  return (vec2i_t){.x = v1.x * v2.x, .y = v1.y * v2.y};
}

vec2i_t vec_mul_num_vec2i_t(vec2i_t v1, i32 num) {
  return (vec2i_t){.x = v1.x * num, .y = v1.y * num};
}

vec2i_t vec_div_num_vec2i_t(vec2i_t v1, i32 num) {
  return (vec2i_t){.x = v1.x / num, .y = v1.y / num};
}

i32 vec_dot_vec2i_t(vec2i_t v1, vec2i_t v2) {
  return v1.x * v2.x + v1.y * v2.y;
}

i32 vec_magnitude_vec2i_t(vec2i_t v) {
  i32 dot_product = vec_dot_vec2i_t(v, v);
  return (i32)sqrtf((f32)dot_product);
}

vec2i_t vec_unit_vec2i_t(vec2i_t v) {
  i32 magnitude = vec_magnitude_vec2i_t(v);
  return (vec2i_t){v.x / magnitude, v.y / magnitude};
}

vec2f_t vec_add_vec2f_t(vec2f_t v1, vec2f_t v2) {
  return (vec2f_t){.x = v1.x + v2.x, .y = v1.y + v2.y};
}

vec2f_t vec_sub_vec2f_t(vec2f_t v1, vec2f_t v2) {
  return (vec2f_t){.x = v1.x - v2.x, .y = v1.y - v2.y};
}

vec2f_t vec_mul_vec2f_t(vec2f_t v1, vec2f_t v2) {
  return (vec2f_t){.x = v1.x * v2.x, .y = v1.y * v2.y};
}

vec2f_t vec_mul_num_vec2f_t(vec2f_t v1, f32 num) {
  return (vec2f_t){.x = v1.x * num, .y = v1.y * num};
}

vec2f_t vec_div_num_vec2f_t(vec2f_t v1, f32 num) {
  return (vec2f_t){.x = v1.x / num, .y = v1.y / num};
}

f32 vec_dot_vec2f_t(vec2f_t v1, vec2f_t v2) {
  return v1.x * v2.x + v1.y * v2.y;
}

f32 vec_magnitude_vec2f_t(vec2f_t v) {
  f32 dot_product = vec_dot_vec2f_t(v, v);
  return sqrtf(dot_product);
}

vec2f_t vec_unit_vec2f_t(vec2f_t v) {
  f32 magnitude = vec_magnitude_vec2f_t(v);
  return (vec2f_t){v.x / magnitude, v.y / magnitude};
}

vec3i_t vec_add_vec3i_t(vec3i_t v1, vec3i_t v2) {
  return (vec3i_t){.x = v1.x + v2.x, .y = v1.y + v2.y, .z = v1.z + v2.z};
}

vec3i_t vec_sub_vec3i_t(vec3i_t v1, vec3i_t v2) {
  return (vec3i_t){.x = v1.x - v2.x, .y = v1.y - v2.y, .z = v1.z - v2.z};
}

vec3i_t vec_mul_vec3i_t(vec3i_t v1, vec3i_t v2) {
  return (vec3i_t){.x = v1.x * v2.x, .y = v1.y * v2.y, .z = v1.z * v2.z};
}

vec3i_t vec_mul_num_vec3i_t(vec3i_t v1, i32 num) {
  return (vec3i_t){.x = v1.x * num, .y = v1.y * num, .z = v1.z * num};
}

vec3i_t vec_div_num_vec3i_t(vec3i_t v1, i32 num) {
  return (vec3i_t){.x = v1.x / num, .y = v1.y / num, .z = v1.z / num};
}

i32 vec_dot_vec3i_t(vec3i_t v1, vec3i_t v2) {
  return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

i32 vec_magnitude_vec3i_t(vec3i_t v) {
  i32 dot_product = vec_dot_vec3i_t(v, v);
  return (i32)sqrtf((f32)dot_product);
}

vec3i_t vec_unit_vec3i_t(vec3i_t v) {
  i32 magnitude = vec_magnitude_vec3i_t(v);
  return (vec3i_t){v.x / magnitude, v.y / magnitude, v.z / magnitude};
}

vec3f_t vec_add_vec3f_t(vec3f_t v1, vec3f_t v2) {
  return (vec3f_t){.x = v1.x + v2.x, .y = v1.y + v2.y, .z = v1.z + v2.z};
}

vec3f_t vec_sub_vec3f_t(vec3f_t v1, vec3f_t v2) {
  return (vec3f_t){.x = v1.x - v2.x, .y = v1.y - v2.y, .z = v1.z - v2.z};
}

vec3f_t vec_mul_num_vec3f_t(vec3f_t v1, f32 num) {
  return (vec3f_t){.x = v1.x * num, .y = v1.y * num, .z = v1.z * num};
}

vec3f_t vec_div_num_vec3f_t(vec3f_t v1, f32 num) {
  return (vec3f_t){.x = v1.x / num, .y = v1.y / num, .z = v1.z / num};
}

vec3f_t vec_mul_vec3f_t(vec3f_t v1, vec3f_t v2) {
  return (vec3f_t){.x = v1.x * v2.x, .y = v1.y * v2.y, .z = v1.z * v2.z};
}

f32 vec_dot_vec3f_t(vec3f_t v1, vec3f_t v2) {
  return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

f32 vec_magnitude_vec3f_t(vec3f_t v) {
  f32 dot_product = vec_dot_vec3f_t(v, v);
  return sqrtf(dot_product);
}

vec3f_t vec_unit_vec3f_t(vec3f_t v) {
  f32 magnitude = vec_magnitude_vec3f_t(v);
  return (vec3f_t){v.x / magnitude, v.y / magnitude, v.z / magnitude};
}

vec3f_t vec_rotate_vec3f_t(vec3f_t original, vec3f_t rotation) {
  f32 x_rad = RADIANS(rotation.x);
  f32 y_rad = RADIANS(rotation.y);
  f32 z_rad = RADIANS(rotation.z);

  f32 cos_x = cosf(x_rad);
  f32 sin_x = sinf(x_rad);

  f32 cos_y = cosf(y_rad);
  f32 sin_y = sinf(y_rad);

  f32 cos_z = cosf(z_rad);
  f32 sin_z = sinf(z_rad);

  f32 row0_col0 = cos_z * cos_y;
  f32 row0_col1 = cos_z * sin_y * sin_x - sin_z * cos_x;
  f32 row0_col2 = cos_z * sin_y * cos_x + sin_z * sin_x;

  f32 row1_col0 = sin_z * cos_y;
  f32 row1_col1 = sin_z * sin_y * sin_x + cos_z * cos_x;
  f32 row1_col2 = sin_z * sin_y * cos_x - cos_z * sin_x;

  f32 row2_col0 = -sin_y;
  f32 row2_col1 = cos_y * sin_x;
  f32 row2_col2 = cos_y * cos_x;

  mat3x3f_t matrix = {
      .row0 = {row0_col0, row0_col1, row0_col2},
      .row1 = {row1_col0, row1_col1, row1_col2},
      .row2 = {row2_col0, row2_col1, row2_col2},
  };

  // clang-format off
	return (vec3f_t){
		.x = matrix.row0.x * original.x + matrix.row0.y * original.y + matrix.row0.z * original.z,
		.y = matrix.row1.x * original.x + matrix.row1.y * original.y + matrix.row1.z * original.z,
		.z = matrix.row2.x * original.x + matrix.row2.y * original.y + matrix.row2.z * original.z,
	};
  // clang-format on
}
